import React, { useState, useEffect, useCallback } from "react";
import {
  TextField,
  Button,
  Card,
  Grid,
  Typography,
  FormHelperText,
  Autocomplete,
  CircularProgress,
  FormControl,
  InputLabel,
} from "@mui/material";
import { useNavigate, useParams } from "react-router-dom";
import { toast } from "react-hot-toast";
import axios from "axios";
import { isAutheticated } from "src/auth";
import { City, State } from "country-state-city";

const UpdatePrincipalDistributor = () => {
  const navigate = useNavigate();
  const token = isAutheticated();
  const { id } = useParams();

  const [user, setUser] = useState({
    PD_ID: "",
    SBU: "",
    name: "",
    email: "",
    phone: "",
  });

  const [data, setData] = useState({
    street: "",
    city: "",
    state: "",
    postalCode: "",
    country: "India",
    tradeName: "",
    gstNumber: "",
    panNumber: "",
  });

  const [loading, setLoading] = useState(false);
  const [stateOptions, setStateOptions] = useState([]);
  const [cityOptions, setCityOptions] = useState([]);
  const [selectedState, setSelectedState] = useState(null);
  const [selectedCity, setSelectedCity] = useState(null);
  const [currentAddressid, setCurrentAddressid] = useState(null);

  // Fetch User Details
  const getUserDetails = useCallback(async () => {
    try {
      const response = await axios.get(`/api/v1/admin/user/${id}`, {
        headers: {
          Authorization: `Bearer ${token}`,
        },
      });
      // console.log(response.data);
      // setUser(response.data.user);
      setUser((prev) => ({
        ...prev,
        PD_ID: response.data.user.uniqueId,
        SBU: response.data.user.SBU,
        name: response.data.user.name,
        email: response.data.user.email,
        phone: response.data.user.phone,
      }));
    } catch (error) {
      swal({
        title: "Warning",
        text: error.message,
        icon: "error",
        button: "Close",
        dangerMode: true,
      });
    }
  }, [id, token]);
  // Fetch Shipping address of the individual user
  const getUserAddress = useCallback(async () => {
    try {
      const response = await axios.get(
        `/api/shipping/address/user/address/${id}`,
        {
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );
      // console.log(response.data);
      const defaultAddress =
        response.data?.UserShippingAddress.find(
          (address) => address.isDefault
        ) ||
        response.data?.UserShippingAddress[0] ||
        {};
      // console.log(defaultAddress);
      setCurrentAddressid(defaultAddress._id);
      setData((prev) => ({
        ...prev,
        street: defaultAddress.street || "",
        city: defaultAddress.city || "",
        state: defaultAddress.state || "",
        postalCode: defaultAddress.postalCode || "",
        country: defaultAddress.country || "India",
        tradeName: defaultAddress.tradeName || "",
        gstNumber: defaultAddress.gstNumber || "",
        panNumber: defaultAddress.panNumber || "",
      }));
      // Fetch city options based on the state from the backend
      if (defaultAddress.state) {
        const state =
          stateOptions.find(
            (option) => option.label === defaultAddress.state
          ) || null;

        // Set selected state from backend address
        setSelectedState(state);

        // Fetch cities if state is found
        if (state) {
          const cities = City.getCitiesOfState("IN", state.value).map(
            (city) => ({
              label: city.name,
              value: city.name,
            })
          );
          setCityOptions(cities);

          // Set selected city if it exists in the fetched city options
          const city =
            cities.find((option) => option.label === defaultAddress.city) ||
            null;
          setSelectedCity(city); // Set the selected city
        }
      } else {
        setSelectedState(null);
        setSelectedCity(null);
        setCityOptions([]); // Clear city options if no address is provided
      }
    } catch (error) {
      swal({
        title: "Warning",
        text: error.message,
        icon: "error",
        button: "Close",
        dangerMode: true,
      });
    }
  }, [id, token, stateOptions]);
  useEffect(() => {
    getUserAddress();
    getUserDetails();
  }, [id, getUserAddress, getUserDetails]);

  // Fetch states when the component mounts
  useEffect(() => {
    const fetchStates = () => {
      const states = State.getStatesOfCountry("IN").map((state) => ({
        label: state.name,
        value: state.isoCode,
      }));
      setStateOptions(states);
    };
    fetchStates();
  }, []);
  useEffect(() => {
    if (stateOptions.length > 0) {
      getUserAddress();
    }
  }, [stateOptions, getUserAddress]);

  // Fetch cities when a state is selected
  useEffect(() => {
    const fetchCities = () => {
      if (selectedState) {
        const cities = City.getCitiesOfState("IN", selectedState.value).map(
          (city) => ({
            label: city.name,
            value: city.name,
          })
        );
        setCityOptions(cities);
      } else {
        setCityOptions([]); // Clear cities if no state is selected
      }
    };
    fetchCities();
  }, [selectedState]);

  const handleInputChange = (e) => {
    setUser({ ...user, [e.target.name]: e.target.value });
  };

  const handleDataChange = (e) => {
    setData({ ...data, [e.target.name]: e.target.value });
  };

  const handleStateChange = (event, newValue) => {
    setSelectedState(newValue);
    setData((prev) => ({
      ...prev,
      state: newValue ? newValue.label : "",
      city: "",
    }));
    setSelectedCity(null); // Clear city when state changes
    setCityOptions([]); // Reset city options
  };

  const handleCityChange = (event, newValue) => {
    setSelectedCity(newValue);
    setData((prev) => ({
      ...prev,
      city: newValue ? newValue.label : "",
    }));
  };

  const handleFormSubmit = async (e) => {
    e.preventDefault();
    try {
      // Validate input fields
      if (
        !user.PD_ID ||
        !user.name ||
        !user.email ||
        !user.phone ||
        !data.panNumber ||
        !data.tradeName ||
        !data.gstNumber ||
        !data.country ||
        !data.state ||
        !data.city ||
        !data.street ||
        !data.postalCode
      ) {
        throw new Error("Fill all fields!");
      }

      setLoading(true);

      // Attempt to register user
      const userResponse = await axios.put(`/api/v1/user/update/${id}`, {
        ...user,
        role: "principal-Distributor",
      });

      if (userResponse.status === 201 || userResponse.status === 200) {
        // const userId = userResponse.data.userId;
        // console.log(userId);
        // Add address details for the user
        const addressResponse = await axios.patch(
          `/api/shipping/address/update/${currentAddressid}`,
          {
            ...data,
            Name: user.name,
            phoneNumber: user.phone,
            isDefault: true,
            state: data.state,
            city: data.city,
            // state: selectedState.label, // Send selected state label
            // city: selectedCity.label, // Send selected city label
          },
          {
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        );

        setLoading(false);
        if (addressResponse.status === 201) {
          toast.success("Principal Distributor and Address updated Successfully");
          navigate("/principal-distributor");
        }
      }
    } catch (error) {
      setLoading(false);
      if (error.response && error.response?.data) {
        toast.error(error.response?.data.message || "Something went wrong!");
      } else {
        toast.error("Something went wrong!");
      }
    }
  };

  const handleCancel = () => {
    navigate("/principal-distributor");
  };

  return (
    <div>
      <Card
        sx={{ padding: "1rem", marginBottom: "1rem", position: "relative" }}
      >
        <Button
          variant="outlined"
          color="secondary"
          style={{
            fontWeight: "bold",
            textTransform: "capitalize",
            position: "absolute",
            top: "10px",
            right: "10px",
          }}
          onClick={handleCancel}
        >
          Cancel
        </Button>
        <Typography variant="h5" sx={{ mb: 3 }}>
          Update Principal Distributor
        </Typography>
        <form onSubmit={handleFormSubmit}>
          <Typography variant="h5" sx={{ mb: 2 }}>
            Basic Information
          </Typography>
          <Grid container spacing={2} sx={{ mb: 2 }}>
            {/* Principal Distributor ID */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="PD_ID"
                  className="form-label"
                >
                  Principal Distributor ID*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="PD_ID"
                  required
                  type="text"
                  fullWidth
                  name="PD_ID"
                  value={user.PD_ID}
                  variant="outlined"
                  onChange={handleInputChange}
                />
              </Grid>
            </Grid>

            {/* Principal Distributor SBU */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="SBU"
                  className="form-label"
                >
                  SBU*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="SBU"
                  required
                  type="text"
                  fullWidth
                  name="SBU"
                  value={user.SBU}
                  variant="outlined"
                  onChange={handleInputChange}
                />
              </Grid>
            </Grid>

            {/* Name */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="name"
                  className="form-label"
                >
                  Name*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="name"
                  required
                  type="text"
                  fullWidth
                  name="name"
                  value={user.name}
                  variant="outlined"
                  onChange={handleInputChange}
                />
              </Grid>
            </Grid>

            {/* Email */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="email"
                  className="form-label"
                >
                  Email*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="email"
                  required
                  type="email"
                  fullWidth
                  name="email"
                  value={user.email}
                  variant="outlined"
                  onChange={handleInputChange}
                />
              </Grid>
            </Grid>

            {/* Phone Number */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="phone"
                  className="form-label"
                >
                  Phone Number*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="phone"
                  required
                  type="text"
                  fullWidth
                  name="phone"
                  value={user.phone}
                  variant="outlined"
                  onChange={handleInputChange}
                />
              </Grid>
            </Grid>
          </Grid>

          <Typography variant="h5" sx={{ mb: 2 }}>
            Business Details
          </Typography>
          <Grid container spacing={2} sx={{ mb: 2 }}>
            {/* PAN Number */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="panNumber"
                  className="form-label"
                >
                  PAN Number*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="panNumber"
                  required
                  type="text"
                  fullWidth
                  name="panNumber"
                  value={data.panNumber}
                  variant="outlined"
                  onChange={handleDataChange}
                />
              </Grid>
            </Grid>

            {/* Trade Name */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="tradeName"
                  className="form-label"
                >
                  Trade Name*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="tradeName"
                  required
                  type="text"
                  fullWidth
                  name="tradeName"
                  value={data.tradeName}
                  variant="outlined"
                  onChange={handleDataChange}
                />
              </Grid>
            </Grid>

            {/* GST Number */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="gstNumber"
                  className="form-label"
                >
                  GST Number*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="gstNumber"
                  required
                  type="text"
                  fullWidth
                  name="gstNumber"
                  value={data.gstNumber}
                  variant="outlined"
                  onChange={handleDataChange}
                />
              </Grid>
            </Grid>
          </Grid>

          <Typography variant="h5" sx={{ mb: 2 }}>
            Address
          </Typography>
          <Grid container spacing={2} sx={{ mb: 2 }}>
            {/* Country */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="country"
                  className="form-label"
                >
                  Country*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="country"
                  required
                  type="text"
                  fullWidth
                  name="country"
                  value={data.country}
                  variant="outlined"
                  disabled
                />
              </Grid>
            </Grid>

            {/* State */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="state"
                  className="form-label"
                >
                  State*
                </Typography>
              </Grid>
              <Grid item xs={5}>
                <Autocomplete
                  id="state"
                  options={stateOptions}
                  // getOptionLabel={(option) => option.label}
                  value={selectedState}
                  onChange={handleStateChange}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      fullWidth
                      variant="outlined"
                      label="Select State"
                      // error={!selectedState}
                      // helperText={!selectedState ? "Select a state" : null}
                    />
                  )}
                />
              </Grid>
              <Grid item xs={5}>
                <TextField
                  id="state"
                  required
                  type="text"
                  fullWidth
                  name="state"
                  value={data.state}
                  variant="outlined"
                  onChange={handleDataChange}
                />
              </Grid>
            </Grid>

            {/* City */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="city"
                  className="form-label"
                >
                  City*
                </Typography>
              </Grid>
              <Grid item xs={5}>
                <Autocomplete
                  id="city"
                  options={cityOptions}
                  // getOptionLabel={(option) => option.label}
                  value={selectedCity}
                  onChange={handleCityChange}
                  isOptionEqualToValue={(option, value) =>
                    option.value === value.value
                  }
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      fullWidth
                      variant="outlined"
                      label="Select City"
                      // error={!selectedCity}
                      // helperText={!selectedCity ? "Select a city" : null}
                    />
                  )}
                />
              </Grid>
              <Grid item xs={5}>
                <TextField
                  id="city"
                  required
                  type="text"
                  fullWidth
                  name="city"
                  value={data.city}
                  variant="outlined"
                  onChange={handleDataChange}
                />
              </Grid>
            </Grid>

            {/* Street */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="street"
                  className="form-label"
                >
                  Street*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="street"
                  required
                  type="text"
                  fullWidth
                  name="street"
                  value={data.street}
                  variant="outlined"
                  onChange={handleDataChange}
                />
              </Grid>
            </Grid>

            {/* Postal Code */}
            <Grid item xs={12} className="d-flex align-items-center">
              <Grid item xs={2}>
                <Typography
                  variant="body1"
                  htmlFor="postalCode"
                  className="form-label"
                >
                  Postal Code*
                </Typography>
              </Grid>
              <Grid item xs={10}>
                <TextField
                  id="postalCode"
                  required
                  type="text"
                  fullWidth
                  name="postalCode"
                  value={data.postalCode}
                  variant="outlined"
                  onChange={handleDataChange}
                />
              </Grid>
            </Grid>
          </Grid>

          <Button
            type="submit"
            disabled={loading}
            variant="contained"
            color="primary"
          >
            {loading ? (
              <CircularProgress size={24} />
            ) : (
              "Create Principal Distributor"
            )}
          </Button>
        </form>
      </Card>
    </div>
  );
};

export default UpdatePrincipalDistributor;
